﻿Option Strict On

<DebuggerStepThrough()> _
<DebuggerDisplay("{ToString}")> _
Public Structure Vector3

    Private ReadOnly m_x As Double
    Private ReadOnly m_y As Double
    Private ReadOnly m_z As Double

    Private ReadOnly m_LenSq As Double

    Public ReadOnly Property x() As Double
        Get
            Return m_x
        End Get
    End Property

    Public ReadOnly Property y() As Double
        Get
            Return m_y
        End Get
    End Property

    Public ReadOnly Property z() As Double
        Get
            Return m_z
        End Get
    End Property

    Public Sub New(ByVal v As Vector3)
        Me.New(v.x, v.y, v.z)
    End Sub

    Public Sub New(ByVal x As Double, ByVal y As Double, ByVal z As Double)
        m_x = x
        m_y = y
        m_z = z
        m_LenSq = x * x + y * y + z * z
    End Sub

    Public ReadOnly Property LengthSq() As Double
        Get
            Return m_LenSq
        End Get
    End Property

    Public ReadOnly Property Length() As Double
        Get
            Return Math.Sqrt(m_LenSq)
        End Get
    End Property

    Public ReadOnly Property Normalize() As Vector3
        Get
            Return Me / Me.Length
        End Get
    End Property

    Public Shared Function Cross(ByVal a As Vector3, ByVal b As Vector3) As Vector3
        Return New Vector3(a.y * b.z - a.z * b.y, _
                           a.z * b.x - a.x * b.z, _
                           a.x * b.y - a.y * b.x)
    End Function

    Public Shared Operator +(ByVal a As Vector3, ByVal b As Vector3) As Vector3
        Return New Vector3(a.x + b.x, a.y + b.y, a.z + b.z)
    End Operator

    Public Shared Operator -(ByVal a As Vector3, ByVal b As Vector3) As Vector3
        Return New Vector3(a.x - b.x, a.y - b.y, a.z - b.z)
    End Operator

    Public Shared Operator *(ByVal a As Vector3, ByVal b As Vector3) As Double
        Return a.x * b.x + a.y * b.y + a.z * b.z
    End Operator

    Public Shared Operator *(ByVal v As Vector3, ByVal r As Double) As Vector3
        Return New Vector3(v.x * r, v.y * r, v.z * r)
    End Operator

    Public Shared Operator *(ByVal r As Double, ByVal v As Vector3) As Vector3
        Return v * r
    End Operator

    Public Shared Operator /(ByVal v As Vector3, ByVal r As Double) As Vector3
        Return New Vector3(v.x / r, v.y / r, v.z / r)
    End Operator

    Public Shared Operator =(ByVal a As Vector3, ByVal b As Vector3) As Boolean
        Return a.x = b.x AndAlso a.y = b.y AndAlso a.z = b.z
    End Operator

    Public Shared Operator <>(ByVal a As Vector3, ByVal b As Vector3) As Boolean
        Return Not a = b
    End Operator

    Public Overrides Function ToString() As String
        Return String.Format("({0}|{1}|{2})", x, y, z)
    End Function

End Structure